<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services\ConfigTerminalCaixaService;

class ConfTerminalCaixaSelectAmbiente extends CrudWithParameterBag {
    
    protected $util;
    protected $dataSourceName = "vnd80600_configterminalcaixa_select_ambiente";
    protected $dataSourceNameMesa = "vnd80600_configterminalcaixa_select_mesa";
    protected $dataSourceNameArray = array (
        'field_ambiente'               => 'vnd80600_configterminalcaixa_field_ambiente',
        'field_mesa'         => 'vnd80600_configterminalcaixa_field_mesa',
        'cadautoambiente'         => 'vnd80600_configterminalcaixa_cad_automatico_ambiente',
        'cadautomesa'       => 'vnd80600_configterminalcaixa_cad_automatico_mesa',
        'autocad'           => 'vnd80600_configterminalcaixa_autocad_mesa'
    );
    protected $entityManager;
    protected $parameterBagColumns = array ("NRORG", "CDFILIAL", "CDLOJA", "NRCONFTELA", "DTINIVIGENCIA");
    protected $parameterBagColumnsArray = array (
        'field_ambiente'          =>  array("NRORG", "CDFILIAL", "CDSALA", "TCDSALA", "NRCONFTELA", "DTINIVIGENCIA"),
        'field_mesa'           =>  array("NRORG", "CDFILIAL", "CDLOJA", "CDSALA", "NRMESA", "TNRMESA", "NRCONFTELA", "DTINIVIGENCIA", "NRPGCONFTAUX", "NRBUTTONAUX"),
        'cadautoambiente'           =>  array("NRORG", "CDFILIAL", "NRCONFTELA", "NRBUTTON", "TNRBUTTON", "DTINIVIGENCIA"),
        'cadautomesa'           => array("NRORG", "CDFILIAL", "CDLOJA", "CDSALA", "NRCONFTELA", "DTINIVIGENCIA")
    );    
    
    public function __construct ($dataSourceManager,$parameterBag, $entityManager, $util,$ConfigTerminalCaixaService) {
        $this->entityManager = $entityManager;
        $this->util = $util;
        $this->ConfigTerminalCaixaService = $ConfigTerminalCaixaService;
        parent::__construct($dataSourceManager, $parameterBag);
    }
    
    public function findMesa(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameMesa;
        $this->parameterBagColumns = $this->parameterBagColumnsArray['field_mesa'];
        parent::find($request, $response);
    }
    
    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function deleteAmbiente(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $this->ConfigTerminalCaixaService->deleteCascadeTable($row);
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
       try {
            $row = $request->getDataSet()->getRows();
            $params = array(
			':CDFILIAL'      => $row[0]["CDFILIAL"],
			':CDLOJA'      => $row[0]["CDLOJA"],
			':NRORG'  		 =>  $row[0]["NRORG"],
			':NRBUTTON'     =>$row[0]["NRBUTTON"],
			':NRCONFTELA' => $row[0]["NRCONFTELA"],
			':NRPGCONFTELA' => $row[0]["NRPGCONFTELA"],
			':DTINIVIGENCIA' => $row[0]["DTINIVIGENCIA"]
		);
		    if($row[0]['__is_new']) {
		        $query = $this->entityManager->getConnection()->fetchAll(self::FIELD_AMBIENTE, $params);
    		    if(count($query) === 0) {
                    $dataSet = $request->getDataSet();
                    $dataSet->setDataSourceName($this->dataSourceName);
                    $persistedRows = $this->dataSourceManager->persist($dataSet);
                    $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
    		    } else {
    		         $response->addNotification(new Notification("Posição já existente!", Notification::TYPE_ERROR));
                     throw new \Exception("Posição já existente!");
    		    }
		    } else {
	            $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceName);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
		    }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function saveMesa(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameMesa;
        
        try {
            $row = $request->getDataSet()->getRows();
            $params = array(
    			':CDFILIAL'      => $row[0]["CDFILIAL"],
    			':CDLOJA'      => $row[0]["CDLOJA"],
    			':CDSALA'    =>$row[0]['CDSALA'],
    			':NRORG'  		 =>  $row[0]["NRORG"],
    			':NRBUTTON'     =>$row[0]["NRBUTTON"],
    			':NRPGCONFTELA'     =>$row[0]["NRPGCONFTELA"],
    			':NRCONFTELA' => $row[0]["NRCONFTELA"],
    			':DTINIVIGENCIA' => $row[0]["DTINIVIGENCIA"]
    		);
		    if($row[0]['__is_new']) {
		        $query = $this->entityManager->getConnection()->fetchAll(self::FIELD_MESA, $params);
    		    if(count($query) === 0) {
    		        $dataSet = $request->getDataSet();
                    $dataSet->setDataSourceName($this->dataSourceName);
                    $persistedRows = $this->dataSourceManager->persist($dataSet);
                    $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
    		    } else {
    		         $response->addNotification(new Notification("Posição já existente!", Notification::TYPE_ERROR));
                     throw new \Exception("Posição já existente!");
    		    }
		    } else {
	            $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceName);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
		    }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findFieldAmbiente (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['field_ambiente'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['field_ambiente'];
        parent::find($request, $response);
    }
    
    public function findFieldMesa (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['field_mesa'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['field_mesa'];
        parent::find($request, $response);
    }
    
    public function findFieldAmbienteCada (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['cadautoambiente'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['cadautoambiente'];
        
        parent::find($request, $response);
    }
    
    public function findFieldMesaCada (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['cadautomesa'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['cadautomesa'];
        
        parent::find($request, $response);
    }
    
   public function saveAutoCada (DTO\Request\Row $request, DTO\Response $response) {
       $row = $request->getRow();
       $mensagem = $this->ConfigTerminalCaixaService->autoCada($row['NRORG'],$row['NRBUTTON'],$row['NRCONFTELA'],$row['CDFILIAL'],$row['NRPGCONFTELA'],$row['NMMESA'],$row['CDSALA'],$row['NRMESA'], $row['DTINIVIGENCIA']);
       $response->addMessage(new Message($mensagem, Message::TYPE_WARNING));
    }
    
    const FIELD_AMBIENTE = "
      SELECT * FROM(SELECT I.NRORG, C.CDLOJA, I.CDFILIAL, I.NRCONFTELA, C.NMCONFTELA, I.NRPGCONFTELA, I.IDTPBUTTON, I.NRBUTTON, I.DSBUTTON, I.CDIDENTBUTON, S.CDSALA, S.NMSALA,
                           I.NRCOLORTEXT, I.NRCOLORBACK, TO_CHAR(I.NRFONTETEXT) AS NRFONTETEXT, IDTPBUTONAUX, NRBUTTONAUX, NRPGCONFTAUX, I.DTINIVIGENCIA
                      FROM ITMENUCONFTE I, CONFTELA C, SALA S
                     WHERE (I.CDFILIAL = :CDFILIAL)
                       AND (I.NRORG = :NRORG)
                       AND (I.NRCONFTELA = :NRCONFTELA)
                       AND (I.NRORG = C.NRORG)
                       AND (I.NRORG = S.NRORG)
                       AND (I.IDTPBUTTON = '4')
                       AND (I.CDFILIAL = C.CDFILIAL)
                       AND (I.NRCONFTELA = C.NRCONFTELA)
                       AND (I.DTINIVIGENCIA = C.DTINIVIGENCIA)
                       AND (C.CDLOJA = :CDLOJA)
                       AND (S.CDSALA (+) = I.CDIDENTBUTON)
                       AND (S.CDFILIAL(+) = I.CDFILIAL)
                     ORDER BY TO_NUMBER(I.NRPGCONFTELA), I.NRBUTTON) 
           WHERE NRBUTTON = :NRBUTTON
             AND NRPGCONFTELA = :NRPGCONFTELA
             AND DTINIVIGENCIA = :DTINIVIGENCIA
";

    const FIELD_MESA = "
        SELECT * FROM (SELECT M.CDSALA, C.CDLOJA, I.NRORG, I.CDFILIAL, I.NRCONFTELA, C.NMCONFTELA, I.NRPGCONFTELA, I.IDTPBUTTON, I.NRBUTTON, I.NRPGCONFTAUX, I.IDTPBUTONAUX,
                              I.NRBUTTONAUX, I.DSBUTTON, I.CDIDENTBUTON, M.NRMESA, M.NMMESA, I.NRCOLORTEXT, I.NRCOLORBACK, TO_CHAR(I.NRFONTETEXT) AS NRFONTETEXT, I.DTINIVIGENCIA
                         FROM ITMENUCONFTE I, CONFTELA C, MESA M, SALA S
                        WHERE (I.CDFILIAL = :CDFILIAL)
                          AND (I.IDTPBUTTON = '3')
                          AND (I.CDFILIAL = C.CDFILIAL)
                          AND (I.NRCONFTELA = C.NRCONFTELA)
                          AND (C.CDLOJA = :CDLOJA)
                          AND (I.NRORG = :NRORG)
                          AND (I.NRORG = C.NRORG)
                          AND (I.NRORG = M.NRORG)
                          AND (I.NRORG = S.NRORG)
                          AND (I.DTINIVIGENCIA = C.DTINIVIGENCIA)
                          AND (M.NRMESA (+) = I.CDIDENTBUTON)
                          AND (M.CDFILIAL (+) = I.CDFILIAL)
                          AND (M.CDLOJA = C.CDLOJA)
                          AND (M.CDSALA = S.CDSALA)
                          AND S.CDFILIAL = C.CDFILIAL
                          AND (I.NRCONFTELA = :NRCONFTELA)
                        ORDER BY TO_NUMBER(I.NRPGCONFTELA),
                                 I.NRBUTTON) 
                WHERE NRBUTTON = :NRBUTTON 
                  AND CDSALA = :CDSALA
                  AND NRPGCONFTELA = :NRPGCONFTELA
                  AND DTINIVIGENCIA = :DTINIVIGENCIA
    ";
}
