<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services\RelatoriosDiversosService;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\Util\Functions;

class RelDiversos {
    protected $relatoriosDiversos;
    protected $utilService;

    public function __construct (RelatoriosDiversosService $relatoriosDiversos, Utilities\Util $utilService ){
        $this->utilService = $utilService;
        $this->relatoriosDiversos = $relatoriosDiversos;
    }
    
    public function getData(Request\Filter $request, Response $response) {
        $dataSourceName = 'vnd01600_relDiversos';
        $result = [];
        try{
            $conditions = $this->utilService->getParamsAsArray($request);
            
            if(isset($conditions['CDFILIAL'])){
                $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
                
                if(isset($conditions['DTPERIODO'])){
                    $conditions['DTINI'] = $conditions['DTPERIODO'][0];
                    $conditions['DTFIN'] = $conditions['DTPERIODO'][1];
                    unset($conditions['DTPERIODO']);
                }
                
                if($conditions['IDTPRELATORIO'] == 0){
                    $conditions['TCDLOJA'] = 'ZZZZZ';
                    if(isset($conditions['CDLOJA'])){
                        $conditions['CDLOJA'] = implode('_', $conditions['CDLOJA']);
                    } else {
                        $conditions['CDLOJA'] = '';
                        $conditions['TCDLOJA'] = 'T';
                    }
                    $result = $this->relatoriosDiversos->getVND0161H($conditions);
                } else if ($conditions['IDTPRELATORIO'] == 1) {
                    $conditions['TCDTIPORECE'] = 'ZZZZZ';
                    if(isset($conditions['CDTIPORECE'])){
                        $conditions['CDTIPORECE'] = implode('_', $conditions['CDTIPORECE']);
                    } else {
                        $conditions['CDTIPORECE'] = '';
                        $conditions['TCDTIPORECE'] = 'T';
                    }
                    $result = $this->relatoriosDiversos->getVND01620($conditions);
                }
            }
            
            return $response->addDataSet(new DataSet($dataSourceName, $result));
        
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
  