<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\Util\Functions;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services\VendProdUnidPesoTipoConsService;
use Teknisa\MAN\Utilities;
use \Exception as Exception;

class VendProdUnidPesoTipoConsController {
    protected $vendProdUnidPesoTipoConsService;
    protected $utilService;

    public function __construct(VendProdUnidPesoTipoConsService $vendProdUnidPesoTipoConsService, Utilities\Util $utilService) {
        $this->vendProdUnidPesoTipoConsService = $vendProdUnidPesoTipoConsService;
        $this->utilService = $utilService;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);
        
        if($this->validaConditions($conditions)) {
            $result = $this->vendProdUnidPesoTipoConsService->find($conditions);
            
            $response->addDataSet(new DataSet("vnd4121L_vendProdUnidPesoTipoCons", $result));
        }
        
        return $result;
    }

    private function buildParams($conditions) {
        $result = [];
        
        if(!Functions::arrayKeyExists('CDFILIAL', $conditions)) {
            $result['TCDFILIAL'] = 'T';
            $result['P_CDFILIAL'] = 'NULL';
        } else {
            $result['P_CDFILIAL'] = "'" . implode("', '", $conditions['CDFILIAL']) . "'";
            $result['TCDFILIAL'] = null;
        }

        if(!Functions::arrayKeyExists('CDCAIXA', $conditions)) {
            $result['TCDCAIXA'] = 'T';
            $result['P_CDCAIXA'] = 'NULL';
        } else {
            $result['P_CDCAIXA'] = "'" . implode("', '", $conditions['CDCAIXA']) . "'";
            $result['TCDCAIXA'] = null;
        }
        
        if(Functions::arrayKeyExists('DTENTRVENDA', $conditions)) {
            $result['P_DTINICIAL'] = $conditions['DTENTRVENDA'][0];
            $result['P_DTFINAL']   = $conditions['DTENTRVENDA'][1];
        }

        if((Functions::arrayKeyExists('INTERVALISTA', $conditions)) && ($conditions['INTERVALISTA'] == 'I')) {
            $result['CDARVPRODI'] = str_replace('%', '', $conditions['CDARVPRODI']);

            if(empty($conditions['CDARVPRODF'])){
                $result['CDARVPRODF'] = $conditions['CDARVPRODI'];
            } else {
                $result['CDARVPRODF'] = str_replace('%', '', $conditions['CDARVPRODF']);
            }

            $result['CDPRODUTO'] = 'NULL';
            $result['TCDPRODUTO'] = null;
        } else if((Functions::arrayKeyExists('INTERVALISTA', $conditions)) && ($conditions['INTERVALISTA'] == 'L')) {
            $result['CDPRODUTO'] = "'" . implode("', '", $conditions['CDARVPROD']) . "'";
            $result['CDARVPRODI'] = null;
            $result['CDARVPRODF'] = null;
            $result['TCDPRODUTO'] = null;
        } else {
            $result['CDPRODUTO'] = 'NULL';
            $result['CDARVPRODI'] = null;
            $result['CDARVPRODF'] = null;
            $result['TCDPRODUTO'] = 'T';
        }

        return $result;
    }
    
    private function validaConditions($conditions) {
        if((isset($conditions['P_CDFILIAL']))  && (!empty($conditions['P_CDFILIAL']))  &&
           (isset($conditions['P_DTINICIAL'])) && (!empty($conditions['P_DTINICIAL'])) &&
           (isset($conditions['P_DTFINAL']))   && (!empty($conditions['P_DTFINAL']))) {
            return true;
        } else {
            return false;
        }
    }
}